=begin
(c) Copyright TIG 2012
You may not use, copy, modify, or distribute this software for
any purpose, without the express prior agreement of the copyright holder: 
this full copyright notice must appear in all copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
###
  #TIG-divide_angle.rb [put into the Plugins folder and restart Sketchup]
###
Usage:
select two edges that are not parallel and that could intersect if they don't 
already share a vertex.
Right-click to get the context-menu 'Divide Angle...'
Note how this context-menu item only available if the selection is appropriate.
In the dialog enter the number of divisions + OK [default is '2'].
It must be a positive integer [2 NOT 2.0]; otherwise you are prompted to retry.
Also it must be 2 or more; otherwise there's an error message and it exits.
Guide-lines are now drawn through the two selected edges and the division[s] as 
you have specified.
The angle between the edges that is less than 180 degrees is always divided.
###
Donations:
by PayPal.com to info @ revitrev.org
###
Version:
1.0 20120803 First release..
###
=end
###
require 'sketchup.rb'
###
module TIG
	def self.divide_angle()
		model=Sketchup.active_model
		ss=model.selection
		ents=model.active_entities
		e0, e1 = ss.to_a
		l0=e0.line
		l1=e1.line
		v0=l0[1]
		v1=l1[1]
		model.start_operation("Divide Angle")
		c0=ents.add_cline(e0.start.position, v0)
		c1=ents.add_cline(e1.start.position, v1)
		results=inputbox(["Divisions: "], [2], "Divide Angle")
		unless results
			model.abort_operation
			return nil
		end
		di=results[0]
		if di < 2
			UI.messagebox("Divide Angle:\n\nDivisions must be >= 2 !")
			model.abort_operation
			return nil
		end
		p0=Geom::intersect_line_line(l0, l1)
		if p0.distance(e0.start.position)>p0.distance(e0.end.position)
			p00=e0.start.position
		else
			p00=e0.end.position
		end
		if p0.distance(e1.start.position)>p0.distance(e1.end.position)
			p11=e1.start.position
		else
			p11=e1.end.position
		end
		v0=p0.vector_to(p00)
		v1=p0.vector_to(p11)
		an=v0.angle_between(v1)
		(di-1).times{
			tr=Geom::Transformation.rotation(p0, v0.cross(v1), an/di.to_f)
			v0.transform!(tr)
			cd=ents.add_cline(p0, v0)
		}
		model.commit_operation
	end
	def self.divide_angle_ok()
		model=Sketchup.active_model
		ss=model.selection
		if ss.length==2 && ss[0].is_a?(Sketchup::Edge) && ss[1].is_a?(Sketchup::Edge) && Geom::intersect_line_line(ss[0].line, ss[1].line) && ! ss[0].line[1].parallel?(ss[1].line[1])
			return true
		else
			return false
		end
	end
	unless file_loaded?(__FILE__)
		UI.add_context_menu_handler{|menu|
			menu.add_item("Divide Angle..."){self.divide_angle()} if self.divide_angle_ok()
		}#menu
	end#if
file_loaded(__FILE__)
end#module
###
